use serde_json::Value;
use std::{
    env::VarError, io, num::ParseIntError, result::Result as StdResult, string::FromUtf8Error,
};

#[derive(Debug)]
pub enum ChatError {
    IoError(io::Error),
    FromUtf8Error(FromUtf8Error),
    ParseIntError(ParseIntError),
    JsonError(JsonError),
    VarError(VarError),
}

#[derive(Debug)]
pub enum JsonError {
    SerdeError(serde_json::Error),
    ExpectedObject(Value),
    ExpectedArray,
    ExpectedString,
    ExpectedMember(String),
}

impl From<io::Error> for ChatError {
    fn from(value: io::Error) -> Self {
        Self::IoError(value)
    }
}

impl From<FromUtf8Error> for ChatError {
    fn from(value: FromUtf8Error) -> Self {
        Self::FromUtf8Error(value)
    }
}

impl From<JsonError> for ChatError {
    fn from(value: JsonError) -> Self {
        Self::JsonError(value)
    }
}

impl From<VarError> for ChatError {
    fn from(value: VarError) -> Self {
        Self::VarError(value)
    }
}

impl From<ParseIntError> for ChatError {
    fn from(value: ParseIntError) -> Self {
        Self::ParseIntError(value)
    }
}

impl From<serde_json::Error> for JsonError {
    fn from(value: serde_json::Error) -> Self {
        Self::SerdeError(value)
    }
}

pub type Result<T> = StdResult<T, ChatError>;
