mod connect;
mod error;
mod lang;
mod readers;
mod run;
mod status;
mod text;
mod writers;

use std::{collections::HashMap, fmt::Debug, net::TcpStream};

use error::Result;
use lang::{Lang, get_lang};
use status::ServerType;

pub trait ServerState {}

#[derive(Debug)]
pub struct Disconnected;
impl ServerState for Disconnected {}
#[derive(Debug)]
pub struct Status {
    typ: ServerType,
    description: String,
    mods: Option<HashMap<String, String>>,
}
impl ServerState for Status {}
#[derive(Debug)]
#[allow(unused)]
pub struct Connected {
    typ: ServerType,
    description: String,
    uuid: String,
    username: String,
}
impl ServerState for Connected {}

#[derive(Debug)]
pub struct Server<S: ServerState + Debug> {
    stream: TcpStream,
    addr: String,
    port: u16,
    lang: Lang,
    state: S,
}

impl Server<Disconnected> {
    fn new(addr: String, port: u16) -> Result<Self> {
        Ok(Self {
            stream: TcpStream::connect(format!("{addr}:{port}"))?,
            addr,
            port,
            lang: get_lang(),
            state: Disconnected,
        })
    }
}

fn main() -> Result<()> {
    let addr = std::env::var("ADDR")?;
    let port: u16 = std::env::var("PORT")?.parse()?;
    Server::new(addr, port)?.status()?.connect()?.run()
}
