use std::{io::Read, net::TcpStream};

use crate::error::Result;

use varint_rs::VarintReader as _;

pub fn read_string(iter: &mut &[u8]) -> Result<String> {
    let len = iter.read_usize_varint()?;
    let mut buf = vec![0u8; len];
    iter.read_exact(&mut buf)?;
    String::from_utf8(buf).map_err(Into::into)
}

pub fn read_packet<T, F: Fn(&mut &[u8]) -> Result<T>>(
    stream: &mut TcpStream,
    read_body: F,
) -> Result<T> {
    let len = stream.read_usize_varint()?;

    let mut buf = vec![0u8; len];
    stream.read_exact(&mut buf)?;

    let mut packet = &buf[..];
    read_body(&mut packet)
}

pub fn read_plugin_msg<T, F: Fn(&mut &[u8]) -> Result<T>>(
    stream: &mut TcpStream,
    channel: &str,
    read_body: F,
) -> Result<T> {
    read_packet(stream, |packet| {
        let id = packet.read_usize_varint()?;
        assert!(id == 63);

        let channel_read = read_string(packet)?;
        assert_eq!(channel_read, channel);
        let mut buf = [0u8; 2];
        packet.read_exact(&mut buf)?;
        let len = u16::from_be_bytes(buf);
        let mut buf = vec![0u8; len as usize];
        packet.read_exact(&mut buf)?;

        let mut packet = &buf[..];
        read_body(&mut packet)
    })
}

pub fn try_read_plugin_msg<T, F: Fn(&str, &mut &[u8]) -> Result<T>>(
    stream: &mut TcpStream,
    read_body: F,
) -> Result<T> {
    read_packet(stream, |packet| {
        let id = packet.read_usize_varint()?;
        assert!(id == 63);

        let channel_read = read_string(packet)?;
        let mut buf = [0u8; 2];
        packet.read_exact(&mut buf)?;
        let len = u16::from_be_bytes(buf);
        let mut buf = vec![0u8; len as usize];
        packet.read_exact(&mut buf)?;

        let mut packet = &buf[..];
        read_body(&channel_read, &mut packet)
    })
}
