use std::{io::Write, net::TcpStream};

use varint_rs::VarintWriter;

use crate::error::Result;

pub fn write_packet<F: Fn(&mut Vec<u8>) -> Result<()>>(
    stream: &mut TcpStream,
    build_packet: F,
) -> Result<()> {
    let mut buf: Vec<u8> = Vec::new();
    build_packet(&mut buf)?;

    stream.write_usize_varint(buf.len())?;
    stream.write_all(&buf)?;

    Ok(())
}

pub fn write_string<W: Write>(buf: &mut W, val: &str) -> Result<()> {
    buf.write_usize_varint(val.len())?;
    buf.write_all(val.as_bytes())?;
    Ok(())
}

pub fn write_plugin_msg<F: Fn(&mut Vec<u8>) -> Result<()>>(
    stream: &mut TcpStream,
    channel: &str,
    build_body: F,
) -> Result<()> {
    let mut body: Vec<u8> = Vec::new();
    build_body(&mut body)?;

    write_packet(stream, |mut buf| {
        buf.write_u8_varint(0x17)?;
        write_string(&mut buf, channel)?;
        buf.write_all(&(body.len() as u16).to_be_bytes())?;
        buf.write_all(&body)?;
        Ok(())
    })
}
